﻿using FLY.OBJComponents.Client;
using FLY.OBJComponents.IService;
using FLY.Thick.Base.UI;
using FLY.Weight.Client;
using FLY.Weight.Common;
using FLY.Weight.IService;
using FLY.Weight.Server.Model;
using GalaSoft.MvvmLight.Command;
using Misc;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Navigation;
using Unity;

namespace FLY.Weight.UI.Client
{
    /// <summary>
    /// Page_Flow.xaml 的交互逻辑
    /// </summary>
    public partial class PgMixTable : Page
    {
        IUnityContainer container;
        IWeightSystemService weightSystemService;
        WeighterC mWeighter;
        PgMixTableVm viewModel;

        public PgMixTable()
        {
            InitializeComponent();

        }

        [InjectionMethod]
        public void Init(
            IUnityContainer container,
            WeighterC w,
            IWeightSystemService weightSystemService,
            MixBufferCollection mixBufferCollection,
            ParamDictionary paramDictionary)
        {
            this.container = container;
            this.weightSystemService = weightSystemService;
            mWeighter = w;
            
            int item_idx = this.weightSystemService.Items.IndexOf(w);
            var bulkDb = mixBufferCollection.GetMix(item_idx);

            viewModel = new PgMixTableVm();
            viewModel.Init(bulkDb, paramDictionary,mWeighter);

            
            //界面........................................

            InitDataGrid();

            this.DataContext = viewModel;
        }

        void InitDataGrid()
        {
            DataGrid datagrid = gridFlows;
            for (int i = 0; i < mWeighter.Ingredients.Count(); i++)
            {
                // kg
                DataGridTextColumn dgtc = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].Disp"), StringFormat = "{0:F4}" }
                };
                StackPanel sp = new StackPanel() { Orientation = Orientation.Horizontal };
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"{mWeighter.Ingredients[i].Number}#"
                    });
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "kg",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc.Header = sp;
                datagrid.Columns.Add(dgtc);

                // %
                DataGridTextColumn dgtc2 = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].PDisp"), StringFormat = "{0:F2}" }
                };
                StackPanel sp2 = new StackPanel() { Orientation = Orientation.Horizontal };
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"{mWeighter.Ingredients[i].Number}#"
                    });
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "%",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc2.Header = sp2;
                dgtc2.CellStyle = this.FindResource("DataGridCellStyle_Percent") as Style;
                datagrid.Columns.Add(dgtc2);

            }
        }
    }

    public class PgMixTableVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// 当前页面最后一行Id
        /// </summary>
        public long Id { get; private set; } = -1;

        /// <summary>
        /// 最新的Id
        /// </summary>
        public long LastId { get; private set; } = -1;

        /// <summary>
        /// 间隔
        /// </summary>
        public int Interval { get; set; } = 1;

        private int windowSize = 30;
        /// <summary>
        /// 一页显示的行数
        /// </summary>
        public int WindowSize
        {
            get { return windowSize; }
            set
            {
                if (value < 10)
                    value = 30;
                if (windowSize != value)
                {
                    windowSize = value;
                }
            }
        }

        /// <summary>
        /// 当前就是最新的视图
        /// </summary>
        public bool IsNewest => Id == Interval * (LastId / Interval);

        /// <summary>
        /// 数据加载中
        /// </summary>
        public bool IsLoading { get; private set; }




        /// <summary>
        /// 通过时间查找数据
        /// </summary>
        public bool IsSearchByTime { get; set; }

        /// <summary>
        /// 查找时间
        /// </summary>
        public DateTime SearchTime { get; set; }

        /// <summary>
        /// 查找Id
        /// </summary>
        public long SearchId { get; set; }



        public ObservableCollection<Lc_Mix> Values { get; } = new ObservableCollection<Lc_Mix>();

        #region Cmd
        public RelayCommand PreViewCmd { get; }
        public RelayCommand NextViewCmd { get; }

        public RelayCommand ToNewestCmd { get; }

        public RelayCommand SearchCmd { get; }

        #endregion
        IBulkDbMixService bulkDb;
        ParamDictionary paramDictionary;
        public WeighterC Item { get; private set; }
        public PgMixTableVm()
        {
            PreViewCmd = new RelayCommand(PreView);
            NextViewCmd = new RelayCommand(NextView);
            ToNewestCmd = new RelayCommand(ToNewest);
            SearchCmd = new RelayCommand(Search);

        }

        public void Init(
            IBulkDbMixService bulkDb,
            ParamDictionary paramDictionary,
            WeighterC w
            )
        {
            this.bulkDb = bulkDb;
            this.paramDictionary = paramDictionary;
            Item = w;
            //窗口显示数据条数
            this.paramDictionary.SetBinding(this, nameof(WindowSize), 30);

            bulkDb.PropertyChanged += BulkDB_PropertyChanged;

            this.PropertyChanged += CtrlTableViewModel_PropertyChanged;

            Misc.BindingOperations.SetBinding(bulkDb, nameof(bulkDb.LastId), this, nameof(LastId));

            ToNewest();
        }


        private void BulkDB_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(FObjBase.FObjServiceClient.IsConnected))
            {
                if ((bulkDb as FObjBase.FObjServiceClient).IsConnected)
                {
                    ToNewest();
                }
                else
                {
                    IsLoading = false;
                }
            }
        }

        private void CtrlTableViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {

        }

        bool IsConnected
        {
            get
            {
                return (bulkDb as FObjBase.FObjServiceClient).IsConnected;
            }
        }


        void GetTrendReponse(object asyncContext, object retData)
        {
            IsLoading = false;
            var reponse = retData as Pack_GetTrendReponse<Lc_Mix>;

            //向Values 后面添加数据
            if (reponse.Values != null && reponse.Values.Count > 0)
            {
                //从尾向前排的！！！！

                //现在把数据反转，Id从小排到大
                reponse.Values.Reverse();

                Values.Clear();

                for (int i = 0; i < reponse.Values.Count(); i++)
                {
                    Values.Add(reponse.Values[i]);
                }

                //限制Values长度
                int remove_cnt = Values.Count() - WindowSize;
                for (int i = 0; i < remove_cnt; i++)
                    Values.RemoveAt(0);


                Id = reponse.Values.Last().ID;
                SearchId = reponse.Values.Last().ID;
                SearchTime = reponse.Values.Last().Time;
            }
            else
            {
                Values.Clear();
                //没有任何数据
                Id = -1;
            }
        }

        void Search()
        {
            if (IsSearchByTime)
            {
                Search(SearchTime);
            }
            else
            {
                Search(SearchId);
            }
        }

        /// <summary>
        /// WHERE ID 小于 lastId AND ID % graphparam.Interval == 0 ORDER BY ID DESC LIMIT graphparam.Len
        /// </summary>
        /// <param name="id"></param>
        void Search(long id)
        {
            if (id <= 0)
                return;

            if (!IsConnected)
                return;
            if (IsLoading)
                return;

            IsLoading = true;

            //TODO,计算，是否有重叠的数据
            bulkDb.GetTrend(
                new Pack_GetTrendRequest()
                {
                    Id = id,
                    Interval = Interval,
                    Count = WindowSize
                },
                GetTrendReponse, this);
        }

        /// <summary>
        /// WHERE Time 小于 lastTime AND ID % graphparam.Interval == 0 ORDER BY ID DESC LIMIT graphparam.Len
        /// </summary>
        /// <param name="lastId"></param>
        void Search(DateTime time)
        {
            if (!IsConnected)
                return;
            if (IsLoading)
                return;

            IsLoading = true;
            //TODO,计算，是否有重叠的数据
            bulkDb.GetTrend(
                new Pack_GetTrendRequest()
                {
                    Interval = Interval,
                    Count = WindowSize,
                    IsSearchByTime = true,
                    Time = time
                },
                GetTrendReponse, this);
        }

        /// <summary>
        /// 下一页
        /// </summary>
        void PreView()
        {
            Search(Id - WindowSize * Interval);
        }

        /// <summary>
        /// 上一页
        /// </summary>
        void NextView()
        {

            Search(Id + WindowSize * Interval);
        }

        /// <summary>
        /// 显示最新数据
        /// </summary>
        void ToNewest()
        {
            if (!IsConnected)
                return;
            if (IsLoading)
                return;

            IsLoading = true;
            bulkDb.GetTrend(
                new Pack_GetTrendRequest()
                {
                    Id = 0,
                    Interval = Interval,
                    Count = WindowSize
                },
                GetTrendReponse, this);
        }
    }
}
